@@ -20,6 +20,8 @@ package jnr.ffi;
 
 import java.io.File;
 import java.io.FilenameFilter;
+import java.util.ArrayList;
+import java.util.Collections;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
@@ -55,6 +57,8 @@ public abstract class Platform {
         NETBSD,
         /** OpenBSD */
         OPENBSD,
+        /** DragonFly */
+        DRAGONFLY,
         /** Linux */
         LINUX,
         /** Solaris (and OpenSolaris) */
@@ -63,8 +67,12 @@ public abstract class Platform {
         WINDOWS,
         /** IBM AIX */
         AIX,
+        /** IBM i */
+        IBMI,
         /** IBM zOS **/
         ZLINUX,
+        /** MidnightBSD **/
+        MIDNIGHTBSD,
         /** No idea what the operating system is */
         UNKNOWN;
 
@@ -74,7 +82,7 @@ public abstract class Platform {
         }
     }
 
-        /**
+    /**
      * The supported CPU architectures.
      */
     public enum CPU {
@@ -116,6 +124,9 @@ public abstract class Platform {
         /** 64 bit ARM */
         AARCH64,
 
+        /** 64 bit MIPS */
+        MIPS64EL,
+
         /**
          * Unknown CPU architecture.  A best effort will be made to infer architecture
          * specific values such as address and long size.
@@ -136,7 +147,7 @@ public abstract class Platform {
     /**
      * Determines the operating system jffi is running on
      *
-     * @return An member of the <tt>OS</tt> enum.
+     * @return An member of the <code>OS</code> enum.
      */
     private static OS determineOS() {
         String osName = System.getProperty("os.name").split(" ")[0];
@@ -148,22 +159,28 @@ public abstract class Platform {
             return OS.SOLARIS;
         } else if (startsWithIgnoreCase(osName, "aix")) {
             return OS.AIX;
+        } else if (startsWithIgnoreCase(osName, "os400") || startsWithIgnoreCase(osName, "os/400")) {
+            return OS.IBMI;
         } else if (startsWithIgnoreCase(osName, "openbsd")) {
             return OS.OPENBSD;
         } else if (startsWithIgnoreCase(osName, "freebsd")) {
             return OS.FREEBSD;
+        } else if (startsWithIgnoreCase(osName, "dragonfly")) {
+            return OS.DRAGONFLY;
         } else if (startsWithIgnoreCase(osName, "windows")) {
             return OS.WINDOWS;
+        } else if (startsWithIgnoreCase(osName, "midnightbsd")) {
+            return OS.MIDNIGHTBSD;
         } else {
             return OS.UNKNOWN;
         }
     }
 
     /**
-     * Determines the <tt>Platform</tt> that best describes the <tt>OS</tt>
+     * Determines the <code>Platform</code> that best describes the <code>OS</code>
      *
      * @param os The operating system.
-     * @return An instance of <tt>Platform</tt>
+     * @return An instance of <code>Platform</code>
      */
     private static Platform determinePlatform(OS os) {
         switch (os) {
@@ -173,6 +190,8 @@ public abstract class Platform {
                 return new Linux();
             case WINDOWS:
                 return new Windows();
+            case IBMI:
+                return new IbmI();
             case UNKNOWN:
                 return new Unsupported(os);
             default:
@@ -204,6 +223,8 @@ public abstract class Platform {
         } else if (equalsIgnoreCase("x86_64", archString) || equalsIgnoreCase("amd64", archString)) {
             return CPU.X86_64;
         } else if (equalsIgnoreCase("ppc", archString) || equalsIgnoreCase("powerpc", archString)) {
+            if (OS.IBMI.equals(determineOS()))
+                return CPU.PPC64;
             return CPU.PPC;
         } else if (equalsIgnoreCase("ppc64", archString) || equalsIgnoreCase("powerpc64", archString)) {
             if ("little".equals(System.getProperty("sun.cpu.endian"))) {
@@ -216,6 +237,10 @@ public abstract class Platform {
             return CPU.S390X;
         } else if (equalsIgnoreCase("aarch64", archString)) {
             return CPU.AARCH64;
+        } else if (equalsIgnoreCase("arm", archString) || equalsIgnoreCase("armv7l", archString)) {
+            return CPU.ARM;
+        } else if (equalsIgnoreCase("mips64", archString) || equalsIgnoreCase("mips64el", archString)) {
+            return CPU.MIPS64EL;
         }
 
         // Try to find by lookup up in the CPU list
@@ -248,6 +273,9 @@ public abstract class Platform {
             case DARWIN:
                 libpattern = "lib.*\\.(dylib|jnilib)$";
                 break;
+            case IBMI:
+                libpattern = "lib.*\\.(so|a\\(shr.o\\)|a\\(shr_64.o\\)|a|so.[\\.0-9]+)$";
+                break;
             default:
                 libpattern = "lib.*\\.so.*$";
                 break;
@@ -259,8 +287,8 @@ public abstract class Platform {
     }
 
     private static int calculateAddressSize(CPU cpu) {
-        int dataModel = Integer.getInteger("sun.arch.data.model");
-        if (dataModel != 32 && dataModel != 64) {
+        Integer dataModel = Integer.getInteger("sun.arch.data.model");
+        if (dataModel == null || dataModel != 32 && dataModel != 64) {
             switch (cpu) {
                 case I386:
                 case PPC:
@@ -273,6 +301,7 @@ public abstract class Platform {
                 case SPARCV9:
                 case S390X:
                 case AARCH64:
+		case MIPS64EL:
                     dataModel = 64;
                     break;
                 default:
@@ -284,7 +313,7 @@ public abstract class Platform {
     }
 
     /**
-     * Gets the native <tt>Platform</tt>
+     * Gets the native <code>Platform</code>
      *
      * @return The current platform.
      */
@@ -300,7 +329,7 @@ public abstract class Platform {
     /**
      * Gets the current Operating System.
      *
-     * @return A <tt>OS</tt> value representing the current Operating System.
+     * @return A <code>OS</code> value representing the current Operating System.
      */
     public final OS getOS() {
         return os;
@@ -309,14 +338,14 @@ public abstract class Platform {
     /**
      * Gets the current processor architecture the JVM is running on.
      *
-     * @return A <tt>CPU</tt> value representing the current processor architecture.
+     * @return A <code>CPU</code> value representing the current processor architecture.
      */
     public final CPU getCPU() {
         return cpu;
     }
     
     public final boolean isBSD() {
-        return os == OS.FREEBSD || os == OS.OPENBSD || os == OS.NETBSD || os == OS.DARWIN;
+        return os == OS.FREEBSD || os == OS.OPENBSD || os == OS.NETBSD || os == OS.DARWIN || os == OS.DRAGONFLY | os == OS.MIDNIGHTBSD;
     }
     public final boolean isUnix() {
         return os != OS.WINDOWS;
@@ -343,7 +372,23 @@ public abstract class Platform {
     }
 
     /**
-     * Gets the name of this <tt>Platform</tt>.
+     * Returns true if the current platform is little endian
+     * @return true if little endian, false otherwise or if cannot determine
+     */
+    public final boolean isLittleEndian() {
+        return "little".equals(System.getProperty("sun.cpu.endian"));
+    }
+
+    /**
+     * Returns true if the current platform is big endian
+     * @return true if big endian, false otherwise or if cannot determine
+     */
+    public final boolean isBigEndian() {
+        return "big".equals(System.getProperty("sun.cpu.endian"));
+    }
+
+    /**
+     * Gets the name of this <code>Platform</code>.
      *
      * @return The name of this platform.
      */
@@ -362,10 +407,13 @@ public abstract class Platform {
             return "libc.so.6";
         case SOLARIS:
             return "c";
+        case DRAGONFLY:
         case FREEBSD:
+        case MIDNIGHTBSD:
         case NETBSD:
             return "c";
         case AIX:
+        case IBMI:
             return addressSize == 32
                 ? "libc.a(shr.o)"
                 : "libc.a(shr_64.o)";
@@ -410,6 +458,52 @@ public abstract class Platform {
         // Default to letting the system search for it
         return mappedName;
     }
+
+    /**
+     * Searches through a list of directories for a native library.
+     *
+     * @param libName      the base name (e.g. "c") of the library to locate
+     * @param libraryPaths the list of directories to search
+     * @param options      map of {@link LibraryOption}s to customize search behavior
+     *                     such as {@link LibraryOption#PreferCustomPaths}
+     * @return the path of the library
+     */
+    public String locateLibrary(String libName, List<String> libraryPaths, Map<LibraryOption, Object> options) {
+        return locateLibrary(libName, libraryPaths);
+    }
+
+    /**
+     * Returns a list of absolute paths to the found locations of a library with the base name {@code libName},
+     * if the returned list is empty then the library could not be found and will fail to be loaded as a result.
+     *
+     * Even if a library is found, this does not guarantee that it will successfully be loaded, it only guarantees
+     * that the reason for the failure was not that it was not found.
+     *
+     * @param libName         the base name (e.g. "c") of the library to locate
+     * @param additionalPaths additional paths to search, these take precedence over default paths,
+     *                        (as is the behavior in {@link LibraryLoader})
+     *                        pass null to only search in the default paths
+     * @return the list of absolute paths where the library was found
+     */
+    public List<String> libraryLocations(String libName, List<String> additionalPaths) {
+        ArrayList<String> result = new ArrayList<>();
+        ArrayList<String> libDirs = new ArrayList<>();
+        if (additionalPaths != null) libDirs.addAll(additionalPaths); // customPaths first!
+        libDirs.addAll(LibraryLoader.DefaultLibPaths.PATHS);
+
+        // locateLibrary can either give us an absolute path with the version at the end (for Linux)
+        //  or just the name (forwards to mapLibraryName), either way we only want the name, we will
+        //  add the parent later from libDirs
+        String name = new File(locateLibrary(libName, libDirs)).getName();
+        for (String libDir : libDirs) {
+            File libFile = new File(libDir, name);
+            if (libFile.exists()) {
+                result.add(libFile.getAbsolutePath());
+            }
+        }
+        return result;
+    }
+
     private static class Supported extends Platform {
         public Supported(OS os) {
             super(os);
@@ -423,11 +517,9 @@ public abstract class Platform {
     }
 
     private static final class Default extends Supported {
-
         public Default(OS os) {
             super(os);
         }
-
     }
     /**
      * A {@link Platform} subclass representing the MacOS system.
@@ -448,48 +540,45 @@ public abstract class Platform {
             }
             return "lib" + libName + ".dylib";
         }
-        
-        @Override
-        public String getName() {
-            return "Darwin";
-        }
 
     }
-    /**
-     * A {@link Platform} subclass representing the Linux operating system.
-     */
-    static final class Linux extends Supported {
 
-        public Linux() {
-            super(OS.LINUX);
+    static final class IbmI extends Supported {
+        public IbmI() {
+            super(OS.IBMI);
         }
 
+        @Override
+        public String mapLibraryName(String libName) {
+            //
+            // A specific version was requested - use as is for search
+            //
+            if (libPattern.matcher(libName).find()) {
+                return libName;
+            }
+            return "lib" + libName + ".a(shr_64.o)";
+        }
         @Override
         public String locateLibrary(final String libName, List<String> libraryPaths) {
-            Pattern exclude;
-            // there are /libx32 directories in wild on ubuntu 14.04 and the
-            // oracle-java8-installer package
-            if (getCPU() == CPU.X86_64) {
-                exclude = Pattern.compile(".*(lib[a-z]*32|i[0-9]86).*");
-            }
-            else {
-                exclude = Pattern.compile(".*(lib[a-z]*64|amd64|x86_64).*");
-            }
-
             final Pattern versionedLibPattern = Pattern.compile("lib" + libName + "\\.so((?:\\.[0-9]+)*)$");
+            final Pattern dotAorSoPattern = Pattern.compile("lib" + libName + "\\.(a|so)$");
+            List<File> dotAorSoFiles = new java.util.LinkedList<File>();
+
+            List<String> searchPaths = new java.util.LinkedList<String>();
+            searchPaths.addAll(libraryPaths);
+            searchPaths.add("/QOpenSys/pkgs/lib");
+            searchPaths.add("/QOpenSys/usr/lib");
 
             FilenameFilter filter = new FilenameFilter() {
                 public boolean accept(File dir, String name) {
-                    return versionedLibPattern.matcher(name).matches();
+                    return dotAorSoPattern.matcher(name).matches() ||  versionedLibPattern.matcher(name).matches() ;
                 }
             };
-
             Map<String, int[]> matches = new LinkedHashMap<String, int[]>();
-            for (String path : libraryPaths) {
-                if (exclude.matcher(path).matches()) {
+            for (String path : searchPaths) {
+                if(path.toLowerCase(LOCALE).startsWith("/qsys")) {
                     continue;
                 }
-
                 File libraryPath = new File(path);
                 File[] files = libraryPath.listFiles(filter);
                 if (files == null) {
@@ -497,6 +586,10 @@ public abstract class Platform {
                 }
 
                 for (File file : files) {
+                    if (dotAorSoPattern.matcher(file.getName()).matches()) {
+                        dotAorSoFiles.add(file);
+                        continue;
+                    }
                     Matcher matcher = versionedLibPattern.matcher(file.getName());
                     String versionString = matcher.matches() ? matcher.group(1) : "";
                     int[] version;
@@ -523,13 +616,141 @@ public abstract class Platform {
                 String file = entry.getKey();
                 int[] fileVersion = entry.getValue();
 
-                if (compareVersions(fileVersion, bestVersion) > 0) {
+                if (Linux.compareVersions(fileVersion, bestVersion) > 0) {
                     bestMatch = file;
                     bestVersion = fileVersion;
                 }
             }
+            if (null != bestMatch) {
+                return bestMatch;
+            }
+            if (!dotAorSoFiles.isEmpty()) {
+                String qualifiedAorSo = dotAorSoFiles.get(0).getAbsolutePath();
+                if(qualifiedAorSo.endsWith(".a")) {
+                    qualifiedAorSo +="(shr_64.o)";
+                }
+                return qualifiedAorSo;
+            }
 
-            return bestMatch != null ? bestMatch : mapLibraryName(libName);
+            return mapLibraryName(libName);
+        }
+    }
+
+    /**
+     * A {@link Platform} subclass representing the Linux operating system.
+     */
+    static final class Linux extends Supported {
+
+        // represents a valid library file that matches the search
+        private static class Match implements Comparable<Match> {
+            String path; // absolute path of library file
+            int[] version; // version of library, empty if no version specified
+            boolean isCustom; // if path is from a custom searchPath specified by user and not default path
+
+            @Override
+            public int compareTo(Match o) { // for Collections.sort() to work
+                return compareVersions(o.version, this.version);
+            }
+        }
+
+        public Linux() {
+            super(OS.LINUX);
+        }
+
+        @Override
+        public String locateLibrary(String libName, List<String> libraryPaths) {
+            return locateLibrary(libName, libraryPaths, null);
+        }
+
+        @Override
+        public String locateLibrary(final String libName, List<String> libraryPaths,
+                                    Map<LibraryOption, Object> options) {
+            List<Match> matches = getMatches(libName, libraryPaths);
+            if (matches.isEmpty()) return mapLibraryName(libName); // no matches, default behavior returns mapped name
+
+            boolean preferCustom = options != null && options.containsKey(LibraryOption.PreferCustomPaths);
+
+            Collections.sort(matches); // sort by version, regardless of location
+
+            Match best = null;
+            if (preferCustom) {
+                for (Match match : matches) {
+                    if (match.isCustom) {
+                        best = match;
+                        break;
+                    }
+                }
+            }
+            return best != null ? best.path : matches.get(0).path;
+        }
+
+        private List<Match> getMatches(String libName, List<String> libraryPaths) {
+            List<String> customPaths = new ArrayList<>();
+            if (LibraryLoader.DefaultLibPaths.PATHS.size() > 0 &&
+                    libraryPaths.size() >= LibraryLoader.DefaultLibPaths.PATHS.size()) {
+                // we were probably called by JNR-FFI, customs will always be before system paths
+                String firstSystemPath = LibraryLoader.DefaultLibPaths.PATHS.get(0);
+
+                // everything before last occurrence of first system path is custom
+                int firstSystemPathIndex = libraryPaths.lastIndexOf(firstSystemPath);
+                for (int i = 0; i < firstSystemPathIndex; i++) {
+                    customPaths.add(libraryPaths.get(i));
+                }
+            } else {
+                // we were probably called by user and not by JNR-FFI, assume all paths are custom
+                customPaths.addAll(libraryPaths);
+            }
+
+            Pattern exclude;
+            // there are /libx32 directories in wild on ubuntu 14.04 and the
+            // oracle-java8-installer package
+            if (getCPU() == CPU.X86_64) {
+                exclude = Pattern.compile(".*(lib[a-z]*32|i[0-9]86).*"); // ignore 32 bit libs on 64-bit
+            } else {
+                exclude = Pattern.compile(".*(lib[a-z]*64|amd64|x86_64).*"); // ignore 64 bit libs on 32-bit
+            }
+
+            final Pattern versionedLibPattern = Pattern.compile("lib" + libName + "\\.so((?:\\.[0-9]+)*)$");
+
+            FilenameFilter filter = new FilenameFilter() {
+                public boolean accept(File dir, String name) {
+                    return versionedLibPattern.matcher(name).matches();
+                }
+            };
+
+            List<Match> matches = new ArrayList<>();
+            for (String path : libraryPaths) {
+                if (exclude.matcher(path).matches()) {
+                    continue;
+                }
+
+                File libraryPath = new File(path);
+                File[] files = libraryPath.listFiles(filter);
+                if (files == null) {
+                    continue;
+                }
+
+                for (File file : files) {
+                    Matcher matcher = versionedLibPattern.matcher(file.getName());
+                    String versionString = matcher.matches() ? matcher.group(1) : "";
+                    int[] version;
+                    if (versionString == null || versionString.isEmpty()) {
+                        version = new int[0];
+                    } else {
+                        String[] parts = versionString.split("\\.");
+                        version = new int[parts.length - 1];
+                        for (int i = 1; i < parts.length; i++) {
+                            version[i - 1] = Integer.parseInt(parts[i]);
+                        }
+                    }
+                    Match match = new Match();
+                    match.path = file.getAbsolutePath();
+                    match.version = version;
+                    match.isCustom = customPaths.contains(path);
+                    matches.add(match);
+                }
+            }
+            return matches;
         }
 
         private static int compareVersions(int[] version1, int[] version2) {
@@ -552,13 +773,7 @@ public abstract class Platform {
             }
 
             // If all components are equal, version with fewest components is smallest
-            if (version1.length < version2.length) {
-                return -1;
-            } else if (version1.length > version2.length) {
-                return 1;
-            } else {
-                return 0;
-            }
+            return Integer.compare(version1.length, version2.length);
         }
 
         @Override
