@@ -133,8 +133,9 @@ public abstract class Struct {
     }
 
     /**
-     * Creates a new <tt>Struct</tt>.
+     * Creates a new <code>Struct</code>.
      *
+     * @param runtime the current jnr-ffi runtime
      * @param isUnion if this Struct is a Union
      */
     protected Struct(Runtime runtime, final boolean isUnion) {
@@ -223,7 +224,13 @@ public abstract class Struct {
             try {
                 sb.append(fieldPrefix);
                 sb.append(field.getName()).append(" = ");
-                sb.append(field.get(this).toString());
+                try {
+                    sb.append(field.get(this).toString());
+                } catch (NullPointerException ex) {
+                    sb.append("- null -");
+                } catch (IllegalAccessException ex) {
+                    sb.append("- IllegalAccessException -");
+                }
                 sb.append("\n");
             } catch (Throwable ex) {
                 throw new RuntimeException(ex);
@@ -325,9 +332,9 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Member</tt> instances.
+     * Creates an array of <code>Member</code> instances.
      *
-     * @param <T> The type of the <tt>Member</tt> subclass to create.
+     * @param <T> The type of the <code>Member</code> subclass to create.
      * @param array the array to store the instances in
      * @return the array that was passed in
      */
@@ -349,11 +356,11 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Enum8</tt> instances.
+     * Creates an array of <code>Enum8</code> instances.
      *
      * @param array     the array to store the instances in
-     * @param enumClass class of <tt>java.lang.Enum</tt>, these <tt>Enum8</tt> instances will represent
-     * @param <T>       The type of the <tt>java.lang.Enum</tt>
+     * @param enumClass class of <code>java.lang.Enum</code>, these <code>Enum8</code> instances will represent
+     * @param <T>       The type of the <code>java.lang.Enum</code>
      * @return the array that was passed in
      */
     protected <T extends java.lang.Enum<T>> Enum8<T>[] array(Enum8<T>[] array, Class<T> enumClass) {
@@ -366,11 +373,11 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Enum16</tt> instances.
+     * Creates an array of <code>Enum16</code> instances.
      *
      * @param array     the array to store the instances in
-     * @param enumClass class of <tt>java.lang.Enum</tt>, these <tt>Enum16</tt> instances will represent
-     * @param <T>       The type of the <tt>java.lang.Enum</tt>
+     * @param enumClass class of <code>java.lang.Enum</code>, these <code>Enum16</code> instances will represent
+     * @param <T>       The type of the <code>java.lang.Enum</code>
      * @return the array that was passed in
      */
     protected <T extends java.lang.Enum<T>> Enum16<T>[] array(Enum16<T>[] array, Class<T> enumClass) {
@@ -383,11 +390,11 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Enum32</tt> instances.
+     * Creates an array of <code>Enum32</code> instances.
      *
      * @param array     the array to store the instances in
-     * @param enumClass class of <tt>java.lang.Enum</tt>, these <tt>Enum32</tt> instances will represent
-     * @param <T>       The type of the <tt>java.lang.Enum</tt>
+     * @param enumClass class of <code>java.lang.Enum</code>, these <code>Enum32</code> instances will represent
+     * @param <T>       The type of the <code>java.lang.Enum</code>
      * @return the array that was passed in
      */
     protected <T extends java.lang.Enum<T>> Enum32<T>[] array(Enum32<T>[] array, Class<T> enumClass) {
@@ -400,11 +407,11 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Enum64</tt> instances.
+     * Creates an array of <code>Enum64</code> instances.
      *
      * @param array     the array to store the instances in
-     * @param enumClass class of <tt>java.lang.Enum</tt>, these <tt>Enum64</tt> instances will represent
-     * @param <T>       The type of the <tt>java.lang.Enum</tt>
+     * @param enumClass class of <code>java.lang.Enum</code>, these <code>Enum64</code> instances will represent
+     * @param <T>       The type of the <code>java.lang.Enum</code>
      * @return the array that was passed in
      */
     protected <T extends java.lang.Enum<T>> Enum64<T>[] array(Enum64<T>[] array, Class<T> enumClass) {
@@ -417,11 +424,11 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Enum</tt> instances.
+     * Creates an array of <code>Enum</code> instances.
      *
      * @param array     the array to store the instances in
-     * @param enumClass class of <tt>java.lang.Enum</tt>, these <tt>Enum</tt> instances will represent
-     * @param <T>       The type of the <tt>java.lang.Enum</tt>
+     * @param enumClass class of <code>java.lang.Enum</code>, these <code>Enum</code> instances will represent
+     * @param <T>       The type of the <code>java.lang.Enum</code>
      * @return the array that was passed in
      */
     protected <T extends java.lang.Enum<T>> Enum<T>[] array(Enum<T>[] array, Class<T> enumClass) {
@@ -434,7 +441,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Struct</tt> instances.
+     * Creates an array of <code>Struct</code> instances.
      *
      * @param array the array to store the instances in
      * @param <T> the type of Struct
@@ -457,7 +464,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Signed8</tt> instances.
+     * Creates an array of <code>Signed8</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -472,7 +479,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Unsigned8</tt> instances.
+     * Creates an array of <code>Unsigned8</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -487,7 +494,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Signed16</tt> instances.
+     * Creates an array of <code>Signed16</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -502,7 +509,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Unsigned16</tt> instances.
+     * Creates an array of <code>Unsigned16</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -517,7 +524,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Signed32</tt> instances.
+     * Creates an array of <code>Signed32</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -532,7 +539,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Unsigned32</tt> instances.
+     * Creates an array of <code>Unsigned32</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -547,7 +554,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Signed64</tt> instances.
+     * Creates an array of <code>Signed64</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -562,7 +569,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Unsigned64</tt> instances.
+     * Creates an array of <code>Unsigned64</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -577,7 +584,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>SignedLong</tt> instances.
+     * Creates an array of <code>SignedLong</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -592,7 +599,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>UnsignedLong</tt> instances.
+     * Creates an array of <code>UnsignedLong</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -607,7 +614,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Float</tt> instances.
+     * Creates an array of <code>Float</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -622,7 +629,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Double</tt> instances.
+     * Creates an array of <code>Double</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -637,7 +644,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Address</tt> instances.
+     * Creates an array of <code>Address</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -652,7 +659,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>Pointer</tt> instances.
+     * Creates an array of <code>Pointer</code> instances.
      *
      * @param array the array to store the instances in
      * @return the array that was passed in
@@ -667,7 +674,7 @@ public abstract class Struct {
     }
 
     /**
-     * Creates an array of <tt>UTF8String</tt> instances.
+     * Creates an array of <code>UTF8String</code> instances.
      *
      * @param array        the array to store the instances in
      * @param stringLength length of each string in array
@@ -721,9 +728,9 @@ public abstract class Struct {
         }
 
         /**
-         * Gets the <tt>Struct</tt> this <tt>Member</tt> is a member of.
+         * Gets the <code>Struct</code> this <code>Member</code> is a member of.
          *
-         * @return a <tt>Struct</tt>.
+         * @return a <code>Struct</code>.
          */
         public final Struct struct() {
             return Struct.this;
@@ -783,7 +790,7 @@ public abstract class Struct {
         }
 
         public final boolean get() {
-            return (getMemory().getByte(offset()) & 0x1) != 0;
+            return getMemory().getByte(offset()) != 0;
         }
 
         public final void set(boolean value) {
@@ -800,7 +807,7 @@ public abstract class Struct {
         }
 
         public final boolean get() {
-            return (getMemory().getInt(offset()) & 0x1) != 0;
+            return getMemory().getInt(offset()) != 0;
         }
 
         public final void set(boolean value) {
@@ -814,7 +821,7 @@ public abstract class Struct {
     }
 
     public final boolean get() {
-      return (getMemory().getShort(offset()) & 0x1) != 0;
+      return getMemory().getShort(offset()) != 0;
     }
 
     public final void set(boolean value) {
@@ -863,7 +870,7 @@ public abstract class Struct {
      */
     public abstract class NumberField extends Member {
         /**
-         * Offset from the start of the <tt>Struct</tt> memory this field is located at.
+         * Offset from the start of the <code>Struct</code> memory this field is located at.
          */
         private final int offset;
         protected final Type type;
@@ -895,9 +902,9 @@ public abstract class Struct {
 
 
         /**
-         * Gets the <tt>Struct</tt> this <tt>Member</tt> is in.
+         * Gets the <code>Struct</code> this <code>Member</code> is in.
          *
-         * @return a <tt>Struct</tt>.
+         * @return a <code>Struct</code>.
          */
         public final Struct struct() {
             return Struct.this;
@@ -918,52 +925,52 @@ public abstract class Struct {
         public abstract void set(java.lang.Number value);
 
         /**
-         * Returns an {@code float} representation of this <tt>Number</tt>.
+         * Returns an {@code float} representation of this <code>Number</code>.
          *
-         * @return an {@code float} value for this <tt>Number</tt>.
+         * @return an {@code float} value for this <code>Number</code>.
          */
         public double doubleValue() {
             return (double) longValue();
         }
 
         /**
-         * Returns an {@code float} representation of this <tt>Number</tt>.
+         * Returns an {@code float} representation of this <code>Number</code>.
          *
-         * @return an {@code float} value for this <tt>Number</tt>.
+         * @return an {@code float} value for this <code>Number</code>.
          */
         public float floatValue() {
             return (float) intValue();
         }
 
         /**
-         * Returns a {@code byte} representation of this <tt>Number</tt>.
+         * Returns a {@code byte} representation of this <code>Number</code>.
          *
-         * @return a {@code byte} value for this <tt>Number</tt>.
+         * @return a {@code byte} value for this <code>Number</code>.
          */
         public byte byteValue() {
             return (byte) intValue();
         }
 
         /**
-         * Returns a {@code short} representation of this <tt>Number</tt>.
+         * Returns a {@code short} representation of this <code>Number</code>.
          *
-         * @return a {@code short} value for this <tt>Number</tt>.
+         * @return a {@code short} value for this <code>Number</code>.
          */
         public short shortValue() {
             return (short) intValue();
         }
 
         /**
-         * Returns a {@code int} representation of this <tt>Number</tt>.
+         * Returns a {@code int} representation of this <code>Number</code>.
          *
-         * @return a {@code int} value for this <tt>Number</tt>.
+         * @return a {@code int} value for this <code>Number</code>.
          */
         public abstract int intValue();
 
         /**
-         * Returns a {@code long} representation of this <tt>Number</tt>.
+         * Returns a {@code long} representation of this <code>Number</code>.
          *
-         * @return a {@code long} value for this <tt>Number</tt>.
+         * @return a {@code long} value for this <code>Number</code>.
          */
         public long longValue() {
             return intValue();
@@ -1802,7 +1809,7 @@ public abstract class Struct {
     public class Address extends NumberField {
 
         /**
-         * Creates a new <tt>Address</tt> field.
+         * Creates a new <code>Address</code> field.
          */
         public Address() {
             super(NativeType.ADDRESS);
@@ -1865,8 +1872,9 @@ public abstract class Struct {
 
 
     public abstract class PointerField extends NumberField {
+        private jnr.ffi.Pointer finalPointer;
         /**
-         * Creates a new <tt>Address</tt> field.
+         * Creates a new <code>Address</code> field.
          */
         public PointerField() {
             super(NativeType.ADDRESS);
@@ -1900,7 +1908,7 @@ public abstract class Struct {
          * @param value the value to write.
          */
         public final void set(jnr.ffi.Pointer value) {
-            jnr.ffi.Pointer finalPointer = value;
+            finalPointer = value;
             if (value instanceof ArrayMemoryIO) {
                 ArrayMemoryIO arrayMemory = (ArrayMemoryIO) value;
                 byte[] valueArray = arrayMemory.array();
@@ -1950,7 +1958,7 @@ public abstract class Struct {
      */
     public class Pointer extends PointerField {
         /**
-         * Creates a new <tt>Address</tt> field.
+         * Creates a new <code>Address</code> field.
          */
         public Pointer() {
         }
@@ -2082,6 +2090,7 @@ public abstract class Struct {
         }
 
         /**
+         * @param length the length of struct in memory to get
          * @return struct from memory
          */
         public final T[] get(int length) {
@@ -2167,7 +2176,7 @@ public abstract class Struct {
         /**
          * Sets the native integer value using a java Enum value.
          *
-         * @param value the java <tt>Enum</tt> value.
+         * @param value the java <code>Enum</code> value.
          */
         public final void set(E value) {
             getMemory().putByte(offset(), (byte) EnumMapper.getInstance(enumClass).intValue(value));
@@ -2577,6 +2586,21 @@ public abstract class Struct {
         public time_t(Offset offset) { super(TypeAlias.time_t, offset); }
     }
 
+    public final class cc_t extends IntegerAlias {
+        public cc_t() { super(TypeAlias.cc_t); }
+        public cc_t(Offset offset) { super(TypeAlias.cc_t, offset); }
+    }
+
+    public final class speed_t extends IntegerAlias {
+        public speed_t() { super(TypeAlias.speed_t); }
+        public speed_t(Offset offset) { super(TypeAlias.speed_t, offset); }
+    }
+
+    public final class tcflag_t extends IntegerAlias {
+        public tcflag_t() { super(TypeAlias.tcflag_t); }
+        public tcflag_t(Offset offset) { super(TypeAlias.tcflag_t, offset); }
+    }
+
     public final class fsblkcnt_t extends IntegerAlias {
         public fsblkcnt_t() { super(TypeAlias.fsblkcnt_t); }
         public fsblkcnt_t(Offset offset) { super(TypeAlias.fsblkcnt_t, offset); }
