@@ -1,17 +1,17 @@
 package jnr.ffi;
 
-import java.io.UnsupportedEncodingException;
-import java.nio.charset.Charset;
-
 import jnr.ffi.annotations.Encoding;
 import jnr.ffi.annotations.Meta;
 import jnr.ffi.provider.FFIProvider;
 import jnr.ffi.types.size_t;
+import org.junit.jupiter.api.AfterAll;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Test;
 
-import org.junit.AfterClass;
-import org.junit.Assert;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import java.io.UnsupportedEncodingException;
+import java.nio.charset.Charset;
+
+import static org.junit.jupiter.api.Assertions.assertEquals;
 
 public class VarargsTest {
     public static interface C {
@@ -20,13 +20,13 @@ public class VarargsTest {
 
     static C c;
 
-    @BeforeClass
+    @BeforeAll
     public static void setUpClass() throws Exception {
         LibraryLoader<C> loader = FFIProvider.getSystemProvider().createLibraryLoader(C.class);
         c = loader.load(Platform.getNativePlatform().getStandardCLibraryName());
     }
 
-    @AfterClass
+    @AfterAll
     public static void tearDownClass() throws Exception {
         c = null;
     }
@@ -34,27 +34,27 @@ public class VarargsTest {
     @Test public void testSizeT() {
         Pointer ptr = Runtime.getRuntime(c).getMemoryManager().allocate(5000);
         int size = c.snprintf(ptr, 5000, "%zu", size_t.class, 12345);
-        Assert.assertEquals(5, size);
+        assertEquals(5, size);
         String result = ptr.getString(0, size, Charset.defaultCharset());
-        Assert.assertEquals("12345", result);
+        assertEquals("12345", result);
     }
 
     @Test public void testMetaAscii() throws UnsupportedEncodingException {
         Pointer ptr = Runtime.getRuntime(c).getMemoryManager().allocate(5000);
         int size = c.snprintf(ptr, 5000, "%s", AsciiEncoding.class, "\u7684");
-        Assert.assertEquals(1, size);
+        assertEquals(1, size);
         String result = ptr.getString(0, size, Charset.forName("ASCII"));
         String expected = new String("\u7684".getBytes("ASCII"), "ASCII");
-        Assert.assertEquals(expected, result);
+        assertEquals(expected, result);
     }
 
     @Test public void testMetaUtf8() throws UnsupportedEncodingException {
         Pointer ptr = Runtime.getRuntime(c).getMemoryManager().allocate(5000);
         int size = c.snprintf(ptr, 5000, "%s", UTF8Encoding.class, "\u7684");
-        Assert.assertEquals(3, size);
+        assertEquals(3, size);
         String result = ptr.getString(0, size, Charset.forName("UTF-8"));
         String expected = new String("\u7684".getBytes("UTF-8"), "UTF-8");
-        Assert.assertEquals(expected, result);
+        assertEquals(expected, result);
     }
 
     @Meta
